@@ -7,16 +7,20 @@ from scipy.optimize import minimize
 from scipy.special import logit, expit
 import numpy as np
 import pandas as pd
+import psutil
 from lifelines.fitters.coxph_fitter import CoxPHFitter
 from pandarallel import pandarallel
 from typing import Optional, List, Union
 from matplotlib import colors as mcolors
 from joblib import Parallel, delayed
+import warnings
+warnings.filterwarnings("ignore")
 
 from .examples_utils.generate_simulations_data import generate_quick_start_df
 from .utils import assert_fit
 
 COLORS = list(mcolors.TABLEAU_COLORS.keys())
+WORKERS = psutil.cpu_count(logical=False)
 
 
 class DataExpansionFitter(ExpansionBasedFitter):
@@ -196,7 +200,7 @@ class DataExpansionFitter(ExpansionBasedFitter):
             summary_df.columns = summary_df.iloc[0]
             summary_df = summary_df.iloc[1:].set_index(summary_df.columns[0])
             summary_df.columns = pd.MultiIndex.from_product([[event], summary_df.columns])
-            full_table = pd.concat([full_table, summary_df.iloc[:len(self.covariates)]], axis=1)
+            full_table = pd.concat([full_table, summary_df.iloc[:-len(self.covariates)-1]], axis=1)
         return full_table
 
 
@@ -231,11 +235,12 @@ class TwoStagesFitter(ExpansionBasedFitter):
     def _fit_event_beta(self, expanded_df, event, model=CoxPHFitter, model_kwargs={}, model_fit_kwargs={}):
         # Model fitting for conditional estimation of Beta_j for specific event
         strata_df = expanded_df[self.covariates + [f'j_{event}', self.duration_col]]
-        strata_df[f'{self.duration_col}_copy'] = expanded_df[self.duration_col]
+        strata_df[f'{self.duration_col}_copy'] = np.ones_like(expanded_df[self.duration_col])
+
         beta_j_model = model(**model_kwargs)
         beta_j_model.fit(df=strata_df[self.covariates + [f'{self.duration_col}', f'{self.duration_col}_copy', f'j_{event}']],
-                         duration_col=self.duration_col, event_col=f'j_{event}', strata=f'{self.duration_col}_copy',
-                         **model_fit_kwargs)
+                         duration_col=f'{self.duration_col}_copy', event_col=f'j_{event}', strata=self.duration_col,
+                         **model_fit_kwargs, batch_mode=False)
         return beta_j_model
 
     def _fit_beta(self, expanded_df, events, model=CoxPHFitter, model_kwargs={}, model_fit_kwargs={}):
@@ -255,8 +260,8 @@ class TwoStagesFitter(ExpansionBasedFitter):
             pid_col: str = 'pid',
             x0: Union[np.array, int] = 0,
             fit_beta_kwargs: dict = {},
-            verbose: int = 2
-            ) -> dict:
+            verbose: int = 2,
+            nb_workers: int = WORKERS) -> dict:
         """
         This method fits a model to the discrete data.
 
@@ -277,6 +282,7 @@ class TwoStagesFitter(ExpansionBasedFitter):
                                                     model_fit_kwargs={}  # keywords arguments to pass on to model.fit() method
                                               }
             verbose (int, Optional): The verbosity level of pandaallel
+            nb_workers (int, Optional): The number of workers to pandaallel. If not sepcified, defaults to the number of workers available.
         Returns:
             event_models (dict): Fitted models dictionary. Keys - event names, Values - fitted models for the event.
         """
@@ -287,7 +293,7 @@ class TwoStagesFitter(ExpansionBasedFitter):
             if len(cov_not_in_df) > 0:
                 raise ValueError(f"Error during fit - missing covariates from df: {cov_not_in_df}")
 
-        pandarallel.initialize(verbose=verbose)
+        pandarallel.initialize(verbose=verbose, nb_workers=nb_workers)
         self.events = [c for c in sorted(df[event_type_col].unique()) if c != 0]
         if covariates is None:
             covariates = [col for col in df if col not in [event_type_col, duration_col, pid_col]]
@@ -315,7 +321,8 @@ class TwoStagesFitter(ExpansionBasedFitter):
             n_et = n_jt[n_jt[event_type_col] == event]
             n_et['opt_res'] = n_et.parallel_apply(lambda row: minimize(self._alpha_jt, x0=x0,
                                     args=(df, y_t.loc[row[duration_col]], self.beta_models[event].params_, row['n_jt'],
-                                    row[duration_col])), axis=1)
+                                    row[duration_col]), method='BFGS',
+                                    options={'gtol': 1e-10, 'eps': 1.5e-08, 'maxiter': 200}), axis=1)
             n_et['success'] = n_et['opt_res'].parallel_apply(lambda val: val.success)
             n_et['alpha_jt'] = n_et['opt_res'].parallel_apply(lambda val: val.x[0])
             assert_fit(n_et, self.times, event_type_col=event_type_col, duration_col=duration_col)  # todo move basic input validation before any optimization
